--
-- Hcksler for Autopilot2
--
-- ModAuthor: Headshot XXL (www.team-flash.eu)
-- date: 22.12.2008
--

Haecksler = {};

function Haecksler:new(configFile, positionX, positionY, positionZ, yRot, customMt)

    if Haecksler_mt == nil then
        Haecksler_mt = Class(Haecksler, Vehicle);
    end;

    local mt = customMt;
    if mt == nil then
        mt = Haecksler_mt;
    end;
    local instance = Haecksler:superClass():new(configFile, positionX, positionY, positionZ, yRot, mt);

    local xmlFile = loadXMLFile("TempConfig", configFile);

    local threshingSound = getXMLString(xmlFile, "vehicle.threshingSound#file");
    if threshingSound ~= nil and threshingSound ~= "" then
        instance.threshingSound = createSample("threshingSound");
        loadSample(instance.threshingSound, threshingSound, false);
        instance.threshingSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.threshingSound#pitchOffset"), 1);
        instance.threshingSoundPitchScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.threshingSound#pitchScale"), 0);
        instance.threshingSoundPitchMax = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.threshingSound#pitchMax"), 2.0);
    end;

    instance.attachedCutter = nil;
    local cutterJoint = {};
    cutterJoint.jointTransform = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.cutterAttacherJoint#index"));
    if cutterJoint.jointTransform ~= nil then

        local rotationNode = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.cutterAttacherJoint#rotationNode"));
        if rotationNode ~= nil then
            cutterJoint.rotationNode = rotationNode;
            local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutterAttacherJoint#maxRot"));
            cutterJoint.maxRot = {};
            cutterJoint.maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
            cutterJoint.maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
            cutterJoint.maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));

            x, y, z = getRotation(rotationNode);
            cutterJoint.minRot = {x,y,z};
        end;
        local rotationNode2 = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.cutterAttacherJoint#rotationNode2"));
        if rotationNode2 ~= nil then
            cutterJoint.rotationNode2 = rotationNode2;
            local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, "vehicle.cutterAttacherJoint#maxRot2"));
            cutterJoint.maxRot2 = {};
            cutterJoint.maxRot2[1] = Utils.degToRad(Utils.getNoNil(x, 0));
            cutterJoint.maxRot2[2] = Utils.degToRad(Utils.getNoNil(y, 0));
            cutterJoint.maxRot2[3] = Utils.degToRad(Utils.getNoNil(z, 0));

            x, y, z = getRotation(rotationNode2);
            cutterJoint.minRot2 = {x,y,z};
        end;
        cutterJoint.moveTime = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.cutterAttacherJoint#moveTime"), 0.5)*1000;
        cutterJoint.jointIndex = 0;

        instance.cutterAttacherJoint = cutterJoint;
    end;

    instance.cutterAttacherJointMoveDown = false;

    instance.pipeParticleSystems = {};
    local pipeIndexStr = getXMLString(xmlFile, "vehicle.pipe#index");
	local pipeklappeIndexStr = getXMLString(xmlFile, "vehicle.pipeklappe#index");
	local pipeentladepointIndexStr = getXMLString(xmlFile, "vehicle.pipeentladepoint#index");
    if pipeIndexStr ~= nil and pipeklappeIndexStr ~= nil and pipeentladepointIndexStr ~= nil then
        instance.pipe = Utils.indexToObject(instance.rootNode, pipeIndexStr);
		instance.pipeklappe = Utils.indexToObject(instance.rootNode, pipeklappeIndexStr);
		instance.pipeentladepoint = Utils.indexToObject(instance.rootNode, pipeentladepointIndexStr);			

		
        local posStr = getXMLString(xmlFile, "vehicle.pipeParticleSystem#position");
        local rotStr = getXMLString(xmlFile, "vehicle.pipeParticleSystem#rotation");
        if posStr ~= nil and rotStr ~= nil then		
            local posX, posY, posZ = Utils.getVectorFromString(posStr);
            local rotX, rotY, rotZ = Utils.getVectorFromString(rotStr);
            rotX = Utils.degToRad(rotX);
            rotY = Utils.degToRad(rotY);
            rotZ = Utils.degToRad(rotZ);
			
            local psFile = getXMLString(xmlFile, "vehicle.pipeParticleSystem#file");
            if psFile == nil then
                psFile = "data/vehicles/particleSystems/wheatParticleSystem.i3d";
            end;
            instance.pipeParticleSystemRoot = loadI3DFile(psFile);
            link(instance.pipeklappe, instance.pipeParticleSystemRoot);
            setTranslation(instance.pipeParticleSystemRoot, posX, posY, posZ);
            setRotation(instance.pipeParticleSystemRoot, rotX, rotY, rotZ);
            for i=0, getNumOfChildren(instance.pipeParticleSystemRoot)-1 do
                local child = getChildAt(instance.pipeParticleSystemRoot, i);
                if getClassName(child) == "Shape" then
                    local geometry = getGeometry(child);
                    if geometry ~= 0 then
                        if getClassName(geometry) == "ParticleSystem" then
                            table.insert(instance.pipeParticleSystems, geometry);
                            setEmittingState(geometry, false);
                        end;
                    end;
                end;
            end;
        end;
    end;
	

    instance.pipeLight = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.pipeLight#index"));

    instance.grainTankCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.grainTankCapacity"), 200);
    instance.grainTankUnloadingCapacity = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.grainTankUnloadingCapacity"), 10);
    instance.grainTankCrowded = false;
	
	instance.ertragInLproHA = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.ertragInLproHA"), 20000);


    instance.combineSize = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.combineSize"), 1);
	
	instance.keys = {};
    local i=0;
    while true do
        local baseName = string.format("vehicle.keys.input(%d)", i);
        local inputName = getXMLString(xmlFile, baseName.. "#name");
        if inputName == nil then
            break;
        end;
        local inputKey = getXMLString(xmlFile, baseName.. "#key");
        if Input[inputKey] == nil then
            print("Error: invalid key '" .. inputKey .. "'  for input event '" .. inputName .. "'");
            break;
        end;
        instance.keys[inputName] = Input[inputKey];
        i = i+1;
    end;

    delete(xmlFile);

    instance.pipeOpen = false;
    instance.pipeParticleActivated = false;
    instance.threshingScale = 1;
    instance:setGrainTankFillLevel(0.0);
    instance.drawFillLevel = true;

    return instance;
end;

function Haecksler:delete()

    self:detachCurrentCutter();

    if self.threshingSound ~= nil then
        delete(self.threshingSound);
    end;
    Haecksler:superClass().delete(self);

end;

function Haecksler:keyEvent(unicode, sym, modifier, isDown)
    Haecksler:superClass().keyEvent(self, unicode, sym, modifier, isDown);
	
	if self.isEntered then
		if isDown and sym == Input.KEY_1 then
			self.speed2Level = 1;
		elseif isDown and sym == Input.KEY_2 then
			self.speed2Level = 2;
		elseif isDown and sym == Input.KEY_3 then
			self.speed2Level = 3;
		elseif isDown and sym == Input.KEY_S then
			self.speed2Level = 0;
		end;
	end;
end;

function Haecksler:update(dt, isActive)
    Haecksler:superClass().update(self, dt, isActive);

    if self.attachedCutter ~= nil then
        self.attachedCutter:update(dt);
    end;

    if self.isEntered then
        
        --if self.grainTankFillLevel < self.grainTankCapacity then
            if InputBinding.hasEvent(InputBinding.ACTIVATE_THRESHING) then
                if self.attachedCutter ~= nil then
                    if self.attachedCutter:isReelStarted() then
                        self:stopThreshing();
                    else
                        self:startThreshing();
                    end;
                end;
            end;
        --end;

        if InputBinding.hasEvent(InputBinding.LOWER_IMPLEMENT) then
            if self.attachedCutter ~= nil then
                self.cutterAttacherJointMoveDown = not self.cutterAttacherJointMoveDown;
            end;
        end;
		
    end;


    --if self.isEntered then

        local jointDesc = self.cutterAttacherJoint;
        if jointDesc.jointIndex ~= 0 then
            if jointDesc.rotationNode ~= nil then
                local x, y, z = getRotation(jointDesc.rotationNode);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, jointDesc.maxRot, jointDesc.minRot, 3, jointDesc.moveTime, dt, not self.cutterAttacherJointMoveDown);
                setRotation(jointDesc.rotationNode, unpack(newRot));
                for i=1, 3 do
                    if math.abs(newRot[i] - rot[i]) > 0.001 then
                        jointFrameInvalid = true;
                    end;
                end;
            end;
            if jointDesc.rotationNode2 ~= nil then
                local x, y, z = getRotation(jointDesc.rotationNode2);
                local rot = {x,y,z};
                local newRot = Utils.getMovedLimitedValues(rot, jointDesc.maxRot2, jointDesc.minRot2, 3, jointDesc.moveTime, dt, not self.cutterAttacherJointMoveDown);
                setRotation(jointDesc.rotationNode2, unpack(newRot));
                for i=1, 3 do
                    if math.abs(newRot[i] - rot[i]) > 0.001 then
                        jointFrameInvalid = true;
                    end;
                end;
            end;
            if jointFrameInvalid then
                setJointFrame(jointDesc.jointIndex, 0, jointDesc.jointTransform);
            end;
        end;


		
		self.pipeParticleActivated = false;
	    --if self.pipeOpen then
	        --self.pipeParticleActivated = true;
	        self.trailerFound = 0;
	        local x,y,z = getWorldTranslation(self.pipeentladepoint);
	        raycastClosest(x, y, z, 0, -1, 0, "findTrailerRaycastCallback", 10, self);
	        
	        local trailer = g_currentMission.objectToTrailer[self.trailerFound];
	        if self.trailerFound == 0 or not trailer:allowFillType(Trailer.FILLTYPE_WHEAT) then
	            self.pipeParticleActivated = false;
	        else

	            local deltaLevel = self.grainTankUnloadingCapacity*dt/1000.0;
	            deltaLevel = math.min(deltaLevel, trailer.capacity - trailer.fillLevel);

	            self.grainTankFillLevel = self.grainTankFillLevel-deltaLevel;
	            if self.grainTankFillLevel <= 0.0 then
	                deltaLevel = deltaLevel+self.grainTankFillLevel;
	                self.grainTankFillLevel = 0.0;
	                self.pipeParticleActivated = false;
	            end;
	            if deltaLevel == 0 then
	                self.pipeParticleActivated = false;
	            end;
	            trailer:setFillLevel(trailer.fillLevel+deltaLevel, Trailer.FILLTYPE_WHEAT);
	        end;
	    --end;
		if self.attachedCutter ~= nil and self.attachedCutter.reelStarted and self.attachedCutter.lastArea > 0 then
		
			self.pipeParticleActivated = true;

            local literPerPixel = self.ertragInLproHA / 10000 / (2*2);
            --literPerPixel = literPerPixel*10;
			
			local x,y,z = getWorldTranslation(self.pipeentladepoint);
			raycastClosest(x, y, z, 0, -1, 0, "findTrailerRaycastCallback", 10, self);
			
			local trailer = g_currentMission.objectToTrailer[self.trailerFound];
			if self.trailerFound == 0 or not trailer:allowFillType(Trailer.FILLTYPE_WHEAT) then
				--self.pipeParticleActivated = true;
			else
				deltaLevel = self.attachedCutter.lastArea*literPerPixel*self.threshingScale;
				--if deltaLevel == 0 then
					--self.pipeParticleActivated = true;
				--end;
				trailer:setFillLevel(trailer.fillLevel+deltaLevel, Trailer.FILLTYPE_WHEAT);
			end;
        end;
		Utils.setEmittingState(self.pipeParticleSystems, self.pipeParticleActivated);

		
		if self.trailerFound > 0 and self.stopNoTrailer then
			self.motor.speedLevel = self.speed2Level;
		end;	
		if self.trailerFound == 0 and not self.isEntered and self.autoPilotEnabled then	
			self.motor.speedLevel = 0;
			self.stopNoTrailer = true;
			g_currentMission:addExtraPrintText("Hcksler STOPP: Kein Trailer");
		end;
		
        
        local pipeRotationSpeed = 0.0006;
        local pipeMinRotY = -180*3.1415/180.0;
		local pipeMaxRotY = 180*3.1415/180.0;
        local pipeMinRotX = -33*3.1415/180.0;
		local pipeMaxRotX = 0*3.1415/180.0;
        local pipeXRotationSpeed = 0.0006;
		local pipeklappeMaxRotX = 0*3.1415/180.0;
		local pipeklappeMinRotX = -70*3.1415/180.0;
		local pipeentladepointMin = -2;
		local pipeentladepointMax = -10;
		local pipeentladepointtransSpeed = 0.01;
        if self.pipe ~= nil then
            local x,y,z = getRotation(self.pipe);
            --if self.pipeOpening then
            if Input.isKeyPressed(Input.KEY_KP_4) then --Input[self.YPIPELEFT] then 
                y = y-dt*pipeRotationSpeed;
            elseif Input.isKeyPressed(Input.KEY_KP_6) then
                y = y+dt*pipeRotationSpeed;
            end;
			if y < pipeMinRotY then
                y = pipeMinRotY;
            end;
			if y > pipeMaxRotY then
                y = pipeMaxRotY;
            end;
			
			if Input.isKeyPressed(Input.KEY_KP_8) then		
                x = x+dt*pipeXRotationSpeed;                
            elseif Input.isKeyPressed(Input.KEY_KP_2) then
                x = x-dt*pipeXRotationSpeed;                
            end;
			if x > pipeMaxRotX then
                x = pipeMaxRotX;
            end;
			if x < pipeMinRotX then
                x = pipeMinRotX;
            end;
			
            setRotation(self.pipe, x, y, z);
            --self.pipeOpen = (math.abs(pipeMinRotY-y) < 0.01);
        end;
		if self.pipeklappe ~= nil then
            local x,y,z = getRotation(self.pipeklappe);
			local px,py,pz = getTranslation(self.pipeentladepoint);
			
			if Input.isKeyPressed(Input.KEY_KP_9) then		
                x = x+dt*pipeXRotationSpeed;
				pz = pz-dt*pipeentladepointtransSpeed;         				
            elseif Input.isKeyPressed(Input.KEY_KP_3) then
                x = x-dt*pipeXRotationSpeed; 
				pz = pz+dt*pipeentladepointtransSpeed; 				
            end;
			if x > pipeklappeMaxRotX then
                x = pipeklappeMaxRotX;
            end;
			if x < pipeklappeMinRotX then
                x = pipeklappeMinRotX;
            end;
			if pz < pipeentladepointMax then
                pz = pipeentladepointMax;
            end;
			if pz > pipeentladepointMin then
                pz = pipeentladepointMin;
            end;
			
            setRotation(self.pipeklappe, x, y, z);
			setTranslation(self.pipeentladepoint, px, py, pz);
            --self.pipeOpen = (math.abs(pipeMinRotY-y) < 0.01);
        end;



        if self.motor.speedLevel == 1 then
            self.speedDisplayScale = 0.5;
        elseif self.motor.speedLevel == 2 then
            self.speedDisplayScale = 0.6;
        else
            self.speedDisplayScale = 1.0;
        end;
    --end;
	
	self.firstTimeRun = true;
end;

function Haecksler:attachCutter(cutter)
    if self.attachedCutter == nil then
        -- attach
        local jointDesc = self.cutterAttacherJoint;

        if jointDesc.rotationNode ~= nil then
            setRotation(jointDesc.rotationNode, unpack(jointDesc.maxRot));
        end;
        if jointDesc.rotationNode2 ~= nil then
            setRotation(jointDesc.rotationNode2, unpack(jointDesc.maxRot2));
        end;
        self.cutterAttacherJointMoveDown = false;

        self.attachedCutter = cutter;

        local constr = JointConstructor:new();
        constr:setActors(self.rootNode, cutter.rootNode);
        constr:setJointTransforms(jointDesc.jointTransform, cutter.attacherJoint);
        --constr:setBreakable(20, 10);

        for i=1, 3 do
            constr:setRotationLimit(i-1, 0, 0);
            constr:setTranslationLimit(i-1, true, 0, 0);
        end;

        jointDesc.jointIndex = constr:finalize();

        self.attachedCutter:onAttach(self);
    end;
end;

function Haecksler:detachCurrentCutter()
    if self.attachedCutter ~= nil then

        local jointDesc = self.cutterAttacherJoint;
        removeJoint(jointDesc.jointIndex);
        jointDesc.jointIndex = 0;

        self.attachedCutter:onDetach();
        self.attachedCutter = nil;

        if self.threshingSound ~= nil then
            stopSample(self.threshingSound);
        end;
    end;
end;

function Haecksler:setGrainTankFillLevel(fillLevel)
    self.grainTankFillLevel = fillLevel;
    if self.grainTankFillLevel > self.grainTankCapacity then
        self.grainTankFillLevel = self.grainTankCapacity;
    end;
    if self.grainTankFillLevel < 0 then
        self.grainTankFillLevel = 0;
    end;
end;

function Haecksler:findTrailerRaycastCallback(transformId, x, y, z, distance)

    self.trailerFound = 0;
    if getUserAttribute(transformId, "vehicleType") == 2 then
        self.trailerFound = transformId;
    end;

    return false;

end;

function Haecksler:onLeave()
    Haecksler:superClass().onLeave(self);
    if self.threshingSound ~= nil then
        stopSample(self.threshingSound);
    end;
end;

function Haecksler:draw()
    Haecksler:superClass().draw(self);
    --local percent = self.grainTankFillLevel/self.grainTankCapacity*100;
    --if self.drawFillLevel then
	
        --Haecksler:superClass().drawGrainLevel(self, self.grainTankFillLevel, self.grainTankCapacity, 95);
    --end;

    if self.attachedCutter ~= nil then
        if self.attachedCutter:isReelStarted() then
            g_currentMission:addHelpButtonText(g_i18n:getText("Turn_off_cutter"), InputBinding.ACTIVATE_THRESHING);
        else
            g_currentMission:addHelpButtonText(g_i18n:getText("Turn_on_cutter"), InputBinding.ACTIVATE_THRESHING);
        end;
    end;

    if self.attachedCutter ~= nil then
        self.attachedCutter:draw();
    end;
end;

function Haecksler:startThreshing()
    if self.attachedCutter ~= nil then
        self.attachedCutter:setReelSpeed(0.003);
        self.attachedCutter:onStartReel();
        self.cutterAttacherJointMoveDown = true;
        if self.threshingSound ~= nil then
            setSamplePitch(self.threshingSound, math.min(self.threshingSoundPitchOffset + self.threshingSoundPitchScale*math.abs(self.lastSoundSpeed), self.threshingSoundPitchMax));
            playSample(self.threshingSound, 0, 1, 0);
        end;
    end;
end;

function Haecksler:stopThreshing()
    if self.attachedCutter ~= nil then
        self.attachedCutter:onStopReel();
        self.cutterAttacherJointMoveDown = false;
        if self.threshingSound ~= nil then
            stopSample(self.threshingSound);
        end;
    end;
end;

function Haecksler:handleAttachEvent()
    if g_currentMission.trailerInTipRange ~= nil then
        g_currentMission.trailerInTipRange:toggleTipState();
    elseif g_currentMission.attachableInMountRange ~= nil and self.attacherJoints[g_currentMission.attachableInMountRangeIndex].jointIndex == 0 then
        self:playAttachSound();
        self:attachImplement(g_currentMission.attachableInMountRange, g_currentMission.attachableInMountRangeIndex);
    elseif self:handleAttachTrailerEvent() then
        self:playAttachSound();
        -- nothing todo
    elseif g_currentMission.cutterInMountRange ~= nil and self.attachedCutter == nil then
        local cutter = g_currentMission.cutterInMountRange;
        self:playAttachSound();
        self:attachCutter(cutter);
    elseif self.attachedTrailer ~= nil then
        self:handleDetachTrailerEvent()
        self:playAttachSound();
    elseif self.attachedCutter ~= nil then
        self:playAttachSound();
        self:detachCurrentCutter();
    elseif self.selectedImplement ~= 0 then
        self:playAttachSound();
        self:detachImplement(self.selectedImplement);
    end;
end;
