--
-- CutterWG for Haecksler
--
-- ModAuthor: Headshot XXL (www.team-flash.eu)
-- date: 15.12.2008
--

CutterWG = {};

function CutterWG:new(configFile, positionX, offsetY, positionZ, rotationY, customMt)

    if CutterWG_mt == nil then
        CutterWG_mt = Class(CutterWG);
    end;

    local instance = {};
    if customMt ~= nil then
        setmetatable(instance, customMt);
    else
        setmetatable(instance, CutterWG_mt);
    end;

    instance.configFileName = configFile;

    local xmlFile = loadXMLFile("TempConfig", configFile);

    local rootNode = loadI3DFile(getXMLString(xmlFile, "vehicle.filename"));
    instance.rootNode = getChildAt(rootNode, 0);
    link(getRootNode(), instance.rootNode);
    delete(rootNode);

    --placementCallback = VehiclePlacementCallback:new();
    --raycastClosest(positionX, 250, positionZ, 0, -1, 0, "placementCallback.callback", 300);

    local terrainHeight = getTerrainHeightAtWorldPos(g_currentMission.terrainRootNode, positionX, 300, positionZ);

    setTranslation(instance.rootNode, positionX, terrainHeight+offsetY, positionZ);
    rotate(instance.rootNode, 0, rotationY, 0);
    --setRigidBodyType(instance.rootNode, "Dynamic");

    local indexStr = getXMLString(xmlFile, "vehicle.reel#index");
    --print(Utils.indexToObject(instance.rootNode, indexStr));
    instance.reelNode = Utils.indexToObject(instance.rootNode, indexStr);
    local numCuttingAreas = getXMLInt(xmlFile, "vehicle.cuttingAreas#count");
    instance.cuttingAreas = {}
    for i=1, numCuttingAreas do
        instance.cuttingAreas[i] = {};
        local areanamei = string.format("vehicle.cuttingAreas.cuttingArea" .. "%d", i);
        local x,y,z = getTranslation(getChildAt(instance.rootNode, getXMLInt(xmlFile, areanamei .. "#startIndex")));
        local widthX, widthY, widthZ = getTranslation(getChildAt(instance.rootNode, getXMLInt(xmlFile, areanamei .. "#widthIndex")));
        local heightX, heightY, heightZ = getTranslation(getChildAt(instance.rootNode, getXMLInt(xmlFile, areanamei .. "#heightIndex")));
        instance.cuttingAreas[i].startX = x;
        instance.cuttingAreas[i].startZ = z;
        instance.cuttingAreas[i].width = widthX-x;
        instance.cuttingAreas[i].height = heightZ-z;
    end;

    instance.attacherJoint = Utils.indexToObject(instance.rootNode, getXMLString(xmlFile, "vehicle.attacherJoint#index"));

    instance.threshingParticleSystems = {};
    local psName = "vehicle.threshingParticleSystem";
    Utils.loadParticleSystem(xmlFile, instance.threshingParticleSystems, psName, instance.rootNode, false)


    delete(xmlFile);

    instance.isAttached = false;
    instance.reelStarted = false;

    instance.printRainWarning = false;


    instance.lastArea = 0;

    return instance;
end;

function CutterWG:delete()
    delete(self.rootNode);
end;

function CutterWG:setWorldPosition(x,y,z, xRot,yRot,zRot)
    setTranslation(self.rootNode, x,y,z);
    setRotation(self.rootNode, xRot,yRot,zRot);
end;

function CutterWG:mouseEvent(posX, posY, isDown, isUp, button)

end;

function CutterWG:keyEvent(unicode, sym, modifier, isDown)
end;


function CutterWG:update(dt)

    self.lastArea = 0;
    if self.isAttached then
        if self.reelStarted and self.attacherVehicle.movingDirection >= 0 then
		            
					local realArea = 0;
                    for i=1, table.getn(self.cuttingAreas) do
                        local worldX, worldY, worldZ = localToWorld(self.rootNode, self.cuttingAreas[i].startX, 0, self.cuttingAreas[i].startZ);
                        local worldX2, worldY2, worldZ2 = localToWorld(self.rootNode, self.cuttingAreas[i].startX+self.cuttingAreas[i].width, 0, self.cuttingAreas[i].startZ);
                        local worldX3, worldY3, worldZ3 = localToWorld(self.rootNode, self.cuttingAreas[i].startX, 0, self.cuttingAreas[i].startZ+self.cuttingAreas[i].height);                       
						local spray = Utils.getDensity(g_currentMission.terrainDetailId, g_currentMission.sprayChannel, worldX, worldZ, worldX2, worldZ2, worldX3, worldZ3);
                        local multi = 1;
                        if spray > 0 then
                            multi = 2;
                        end;
                        
						local areaW = 0;
						local areaG = 0;

						Utils.updateCuttedWheatArea(worldX, worldZ, worldX2, worldZ2, worldX3, worldZ3);
						local areaW = Utils.updateWheatArea(worldX, worldZ, worldX2, worldZ2, worldX3, worldZ3, 0.0);
						local areaG = Utils.updateMeadowArea(worldX, worldZ, worldX2, worldZ2, worldX3, worldZ3);
						local areaG = Utils.updateCuttedMeadowArea(worldX, worldZ, worldX2, worldZ2, worldX3, worldZ3);
						self.lastArea = self.lastArea + (areaW*multi) + areaG;
						realArea = realArea + areaW;

                    end;
                    local pixelToQm = 2048 / 4096 * 2048 / 4096; -- 4096px are mapped to 2048m
                    local qm = realArea*pixelToQm;
                    local ha = qm/10000;

                    g_currentMission.missionStats.hectaresThreshedTotal = g_currentMission.missionStats.hectaresThreshedTotal + ha;
                    g_currentMission.missionStats.hectaresThreshedSession = g_currentMission.missionStats.hectaresThreshedSession + ha;

                    g_currentMission.missionStats.threshingDurationTotal = g_currentMission.missionStats.threshingDurationTotal + dt/(1000*60);
                    g_currentMission.missionStats.threshingDurationSession= g_currentMission.missionStats.threshingDurationSession + dt/(1000*60);

        end;

        Utils.setEmittingState(self.threshingParticleSystems, (self.reelStarted and self.lastArea > 0.0));

        if self.reelStarted then
            rotate(self.reelNode, -dt*self.reelSpeed, 0, 0);
        end;
    end;
end


function CutterWG:draw()


end;

function CutterWG:onAttach(attacherVehicle)
    self.isAttached = true;
    self.attacherVehicle = attacherVehicle;
end;

function CutterWG:onDetach()
    self.isAttached = false;
    self:onStopReel();
    self.attacherVehicle = nil;
    Utils.setEmittingState(self.threshingParticleSystems, false);
end;

function CutterWG:setReelSpeed(speed)
    self.reelSpeed = speed;
end;

function CutterWG:onStartReel()
    self.reelStarted = true;
end;

function CutterWG:onStopReel()
    self.reelStarted = false;
    Utils.setEmittingState(self.threshingParticleSystems, false);
end;

function CutterWG:isReelStarted()
    return self.reelStarted;
end;